@@ -21,7 +21,9 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 #include <R_ext/Lapack.h>
 //#include "def.h" // never change this line
-#include <General_utils.h> //#include <General_utils.h>
+#include <General_utils.h>
+#include "dummy.h" //#include <General_utils.h>
+#include "dummy.h"
 #include "xport_import.h"
 
 
